package com.empatnusabangsa.pocdrivingtracking

import android.Manifest
import android.content.pm.PackageManager
import android.os.Bundle
import android.util.Log
import android.widget.Button
import android.widget.TextView
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import com.raxeltelematics.v2.sdk.TrackingApi
import com.raxeltelematics.v2.sdk.server.model.Locale
import com.raxeltelematics.v2.sdk.utils.permissions.PermissionsWizardActivity
import com.telematicssdk.auth.TelematicsAuth
import java.time.LocalDateTime
import java.time.ZoneOffset
import java.time.format.DateTimeFormatter
import java.time.temporal.ChronoUnit
import java.util.*

class MainActivity() : AppCompatActivity() {
    private var tracking = TrackingApi.getInstance()
    private val INSTANCE_ID = "ef7f9424-f36d-4ee1-90ef-603dfd573bfe"
    private val INSTANCE_KEY = "371c9740-1932-4a2e-b256-b0269dba5cdb"
    private var DEVICE_TOKEN = ""
    private var USER_DEVICE_TOKEN = "9593c574-6487-4e08-80b8-bee912c2fcc0"

    private fun initTracking() {
//        tracking = TrackingApi.getInstance();
        tracking.initialize(this);
        Log.d("PERMISSION CHECKING", "INIT PERMISSION CHECKING")
        val activityResultLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {result ->
            Log.d("PERMISSION CODE", result.resultCode.toString())
            if (result.resultCode == PermissionsWizardActivity.WIZARD_RESULT_ALL_GRANTED) {
                enableTracking()
            }
        }

        if (!tracking.isAllRequiredPermissionsAndSensorsGranted()){
            val intent = PermissionsWizardActivity.getStartWizardIntent(
                this,
                enableAggressivePermissionsWizard = true,
                enableAggressivePermissionsWizardPage = true
            )
            activityResultLauncher.launch(intent)
        } else {
            var sharedPreferences = this.getPreferences( MODE_PRIVATE)
            this.DEVICE_TOKEN = sharedPreferences.getString("DEVICE_TOKEN", "").toString()
            tracking.setDeviceID(this.USER_DEVICE_TOKEN)
            tracking.startTracking()
            Log.d("DEVICE TOKEN", this.DEVICE_TOKEN)
            Log.d("USER DEVICE TOKEN", this.USER_DEVICE_TOKEN)

        }
    }

    private fun enableTracking() {
        if(ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.ACCESS_FINE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED
        ){
            Log.d("PERMISSION", "PERMISSION NOT GRANTED")

            return
        }


        Log.d("INITIATE DEVICE TOKEN", "INITIATE DEVICE TOKEN")
        var sharedPreferences = this.getPreferences( MODE_PRIVATE)
        this.DEVICE_TOKEN = sharedPreferences.getString("DEVICE_TOKEN", "").toString()

        if(this.DEVICE_TOKEN.isNullOrEmpty()){

            val uuid = UUID.randomUUID().toString()

            with (sharedPreferences.edit()){
                putString("DEVICE_TOKEN", uuid)
//                putString("ACCESS_TOKEN", it.accessToken)
//                putString("REFRESH_TOKEN", it.refreshToken)
                Log.d("CREATE DEVICE TOKEN", uuid)
//                Log.d("CREATE ACCESS TOKEN", it.accessToken)
//                Log.d("CREATE REFRESH TOKEN", it.refreshToken)

                apply()
            }
            this.DEVICE_TOKEN = sharedPreferences.getString("DEVICE_TOKEN", "").toString()
            tracking.setDeviceID(this.USER_DEVICE_TOKEN)
            tracking.setEnableSdk(true)
            tracking.startTracking()

//            TelematicsAuth.createDeviceToken(
//                instanceId = this.INSTANCE_ID,
//                instanceKey = this.INSTANCE_KEY)
//                .onSuccess {
//                    with (sharedPreferences.edit()){
//                        putString("DEVICE_TOKEN", it.deviceToken)
//                        putString("ACCESS_TOKEN", it.accessToken)
//                        putString("REFRESH_TOKEN", it.refreshToken)
//                        Log.d("CREATE DEVICE TOKEN", it.deviceToken)
//                        Log.d("CREATE ACCESS TOKEN", it.accessToken)
//                        Log.d("CREATE REFRESH TOKEN", it.refreshToken)
//
//                        apply()
//                    }
//                    this.DEVICE_TOKEN = sharedPreferences.getString("DEVICE_TOKEN", "").toString()
//                    tracking.setDeviceID(this.USER_DEVICE_TOKEN)
//                    tracking.setEnableSdk(true)
//                    tracking.startTracking()
//                }
//                .onError { e ->
//                    e.printStackTrace()
//                }
        } else {
            tracking.setDeviceID(this.USER_DEVICE_TOKEN)
            tracking.setEnableSdk(true)
            tracking.startTracking()
        }

        Log.d("DEVICE TOKEN", this.DEVICE_TOKEN)
        Log.d("USER DEVICE TOKEN", this.USER_DEVICE_TOKEN)


    }

    private fun createDeviceToken() {
        Log.d("INITIATE DEVICE TOKEN", "INITIATE DEVICE TOKEN")

        TelematicsAuth.createDeviceToken(
            instanceId = this.INSTANCE_ID,
            instanceKey = this.INSTANCE_KEY)
            .onSuccess {
                DEVICE_TOKEN = it.deviceToken
                Log.d("CREATE DEVICE TOKEN", it.deviceToken)
                Log.d("CREATE ACCESS TOKEN", it.accessToken)
                Log.d("CREATE REFRESH TOKEN", it.refreshToken)
            }
            .onError { e ->
                e.printStackTrace()
            }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        initTracking()

        val buttonShowTracking = findViewById(R.id.button_show_tracking) as Button
        val textTracking = findViewById(R.id.textTracking) as TextView

        buttonShowTracking.setOnClickListener{
            Log.d("DEVICE TOKEN", this.DEVICE_TOKEN)
            Log.d("USER DEVICE TOKEN", this.USER_DEVICE_TOKEN  )
            Log.d("IS TRACKING ENABLE", tracking.isTracking().toString())
            val current = LocalDateTime.now().atOffset(ZoneOffset.UTC)
            val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'+00:00'")
            val yesterday = current.minus(1, ChronoUnit.DAYS)

            Log.d("currenttime","Current Date and Time is: " + current.format(formatter))
            val result = tracking.getTracks(
                    Locale.EN,
                    null,
                null,
                    0,
                    100)
//            val listTripData =
//                tripsMapper.transformTripsList(arrayOfTracks.asList()).filter { !it.isDeleted }
//            tripsMapper.sort(listTripData, tripData)
//            if (listTripData.isNotEmpty()) {
//                tripData = listTripData[0]
//            }
            var logText = "DEVICE ID: " + this.DEVICE_TOKEN + "\n"
            logText += "USER ID: " + this.USER_DEVICE_TOKEN + "\n"
            logText += "TOTAL DATA: " + result.count() + "\n"

            for (dt in result.asList()){
                logText += "Tracking ID: " + dt.trackId.toString() + "\n"
                logText += "tracking: \n" + dt.toString() + "\n"
                logText += "=========================\n\n"
                Log.d("tracking ID", dt.trackId.toString())
                Log.d("tracking", dt.toString())
            }

            textTracking.text = logText

        }
    }
}